@@ -1,3 +1,21 @@
+# vim: set fileencoding=utf-8:
+#
+# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
+#
+# Copyright (c) 2015-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2015-2021 Ben Nuttall <ben@bennuttall.com>
+# Copyright (c) 2019 tuftii <3215045+tuftii@users.noreply.github.com>
+# Copyright (c) 2019 Jeevan M R <14.jeevan@gmail.com>
+# Copyright (c) 2019 ForToffee <ForToffee@users.noreply.github.com>
+# Copyright (c) 2018 Claire Pollard <claire.r.pollard@gmail.com>
+# Copyright (c) 2016 pcopa <scheltovandoorn@gmail.com>
+# Copyright (c) 2016 Ian Harcombe <ian.harcombe@gmail.com>
+# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
+# Copyright (c) 2016 Andrew Scheller <lurch@durge.org>
+# Copyright (c) 2015 Philip Howard <phil@gadgetoid.com>
+#
+# SPDX-License-Identifier: BSD-3-Clause
+
 from __future__ import (
     unicode_literals,
     print_function,
@@ -29,6 +47,7 @@ from .mixins import (
     SourceMixin,
     ValuesMixin,
     EventsMixin,
+    event,
     HoldMixin,
 )
 from .input_devices import (
@@ -40,6 +59,7 @@ from .input_devices import (
     MotionSensor,
     LightSensor,
     DistanceSensor,
+    RotaryEncoder,
 )
 from .spi_devices import (
     SPIDevice,
@@ -68,6 +88,7 @@ from .output_devices import (
     Servo,
     AngularServo,
     RGBLED,
+    TonalBuzzer,
 )
 from .boards import (
     CompositeOutputDevice,
@@ -75,7 +96,11 @@ from .boards import (
     LEDCollection,
     LEDBoard,
     LEDBarGraph,
+    LEDCharDisplay,
+    LEDMultiCharDisplay,
+    LEDCharFont,
     LedBorg,
+    PiHutXmasTree,
     PiLiter,
     PiLiterBarGraph,
     TrafficLights,
@@ -87,16 +112,23 @@ from .boards import (
     TrafficLightsBuzzer,
     FishDish,
     TrafficHat,
+    TrafficpHat,
     Robot,
     RyanteckRobot,
     CamJamKitRobot,
     PhaseEnableRobot,
     PololuDRV8835Robot,
     Energenie,
+    PumpkinPi,
+    JamHat,
+    Pibrella,
 )
-from .other_devices import (
+from .internal_devices import (
     InternalDevice,
+    PolledInternalDevice,
     PingServer,
     CPUTemperature,
+    LoadAverage,
     TimeOfDay,
+    DiskUsage,
 )
