@@ -1,3 +1,14 @@
+# vim: set fileencoding=utf-8:
+#
+# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
+#
+# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2019-2020 Ben Nuttall <ben@bennuttall.com>
+# Copyright (c) 2019 Kosovan Sofiia <sofiia.kosovan@gmail.com>
+# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
+#
+# SPDX-License-Identifier: BSD-3-Clause
+
 from __future__ import (
     unicode_literals,
     print_function,
@@ -25,6 +36,9 @@ class BadQueueLen(GPIOZeroError, ValueError):
 class BadPinFactory(GPIOZeroError, ImportError):
     "Error raised when an unknown pin factory name is specified"
 
+class ZombieThread(GPIOZeroError, RuntimeError):
+    "Error raised when a thread fails to die within a given timeout"
+
 class CompositeDeviceError(GPIOZeroError):
     "Base class for errors specific to the CompositeDevice hierarchy"
 
@@ -67,6 +81,9 @@ class SPIFixedSelect(SPIError, AttributeError):
 class SPIFixedWordSize(SPIError, AttributeError):
     "Error raised when the number of bits per word cannot be changed"
 
+class SPIFixedRate(SPIError, AttributeError):
+    "Error raised when the baud-rate of the interface cannot be changed"
+
 class SPIInvalidWordSize(SPIError, ValueError):
     "Error raised when an invalid (out of range) number of bits per word is specified"
 
@@ -155,7 +172,13 @@ class SPIWarning(GPIOZeroWarning):
     "Base class for warnings related to the SPI implementation"
 
 class SPISoftwareFallback(SPIWarning):
-    "Warning raised when falling back to the software implementation"
+    "Warning raised when falling back to the SPI software implementation"
+
+class PWMWarning(GPIOZeroWarning):
+    "Base class for PWM warnings"
+
+class PWMSoftwareFallback(PWMWarning):
+    "Warning raised when falling back to the PWM software implementation"
 
 class PinWarning(GPIOZeroWarning):
     "Base class for warnings related to pin implementations"
@@ -163,6 +186,17 @@ class PinWarning(GPIOZeroWarning):
 class PinFactoryFallback(PinWarning):
     "Warning raised when a default pin factory fails to load and a fallback is tried"
 
+class NativePinFactoryFallback(PinWarning):
+    "Warning raised when all other default pin factories fail to load and NativeFactory is used"
+
 class PinNonPhysical(PinWarning):
     "Warning raised when a non-physical pin is specified in a constructor"
 
+class ThresholdOutOfRange(GPIOZeroWarning):
+    "Warning raised when a threshold is out of range specified by min and max values"
+
+class CallbackSetToNone(GPIOZeroWarning):
+    "Warning raised when a callback is set to None when its previous value was None"
+
+class AmbiguousTone(GPIOZeroWarning):
+    "Warning raised when a Tone is constructed with an ambiguous number"
