@@ -1,4 +1,13 @@
 # vim: set fileencoding=utf-8:
+#
+# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
+#
+# Copyright (c) 2015-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2018 Rick Ansell <rick@nbinvincible.org.uk>
+# Copyright (c) 2018 Mike Kazantsev <mk.fraggod@gmail.com>
+# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
+#
+# SPDX-License-Identifier: BSD-3-Clause
 
 from __future__ import (
     unicode_literals,
@@ -12,6 +21,7 @@ from weakref import ref
 from collections import defaultdict
 from threading import Lock
 
+from ..devices import Device
 from ..exc import (
     PinInvalidFunction,
     PinSetInput,
@@ -24,6 +34,7 @@ from ..exc import (
     SPIFixedBitOrder,
     SPIFixedSelect,
     SPIFixedWordSize,
+    SPIFixedRate,
     GPIOPinInUse,
     )
 
@@ -31,8 +42,13 @@ from ..exc import (
 class Factory(object):
     """
     Generates pins and SPI interfaces for devices. This is an abstract
-    base class for pin factories. Descendents *may* override the following
-    methods, if applicable:
+    base class for pin factories. Descendents *must* override the following
+    methods:
+
+    * :meth:`ticks`
+    * :meth:`ticks_diff`
+
+    Descendents *may* override the following methods, if applicable:
 
     * :meth:`close`
     * :meth:`reserve_pins`
@@ -65,10 +81,10 @@ class Factory(object):
     def release_pins(self, reserver, *pins):
         """
         Releases the reservation of *reserver* against *pins*.  This is
-        typically called during :meth:`Device.close` to clean up reservations
-        taken during construction. Releasing a reservation that is not currently
-        held will be silently ignored (to permit clean-up after failed / partial
-        construction).
+        typically called during :meth:`~gpiozero.Device.close` to clean up
+        reservations taken during construction. Releasing a reservation that is
+        not currently held will be silently ignored (to permit clean-up after
+        failed / partial construction).
         """
         with self._res_lock:
             for pin in pins:
@@ -82,14 +98,13 @@ class Factory(object):
         Releases all pin reservations taken out by *reserver*. See
         :meth:`release_pins` for further information).
         """
-        with self._res_lock:
-            self._reservations = defaultdict(list, {
-                pin: [
-                    ref for ref in conflictors
-                    if ref() not in (reserver, None)
-                    ]
-                for pin, conflictors in self._reservations.items()
-                })
+        # Yes, this would be more efficient if it simply regenerated the
+        # reservations list without any references to reserver instead of
+        # (in release_pins) looping over each pin individually. However, this
+        # then causes a subtle bug in LocalPiFactory which does something
+        # horribly naughty (with good reason) and makes its _reservations
+        # dictionary equivalent to a class-level one.
+        self.release_pins(reserver, *self._reservations)
 
     def close(self):
         """
@@ -111,7 +126,8 @@ class Factory(object):
             :meth:`pin` for the same pin specification must return the same
             object.
         """
-        raise PinUnsupported("Individual pins are not supported by this pin factory")
+        raise PinUnsupported(  # pragma: no cover
+            "Individual pins are not supported by this pin factory")
 
     def spi(self, **spi_args):
         """
@@ -121,10 +137,33 @@ class Factory(object):
         be used; attempting to mix *port* and *device* with pin numbers will
         raise :exc:`SPIBadArgs`.
         """
-        raise PinSPIUnsupported('SPI not supported by this pin factory')
+        raise PinSPIUnsupported(  # pragma: no cover
+            'SPI not supported by this pin factory')
+
+    def ticks(self):
+        """
+        Return the current ticks, according to the factory. The reference point
+        is undefined and thus the result of this method is only meaningful when
+        compared to another value returned by this method.
+
+        The format of the time is also arbitrary, as is whether the time wraps
+        after a certain duration. Ticks should only be compared using the
+        :meth:`ticks_diff` method.
+        """
+        raise NotImplementedError
+
+    def ticks_diff(self, later, earlier):
+        """
+        Return the time in seconds between two :meth:`ticks` results. The
+        arguments are specified in the same order as they would be in the
+        formula *later* - *earlier* but the result is guaranteed to be in
+        seconds, and to be positive even if the ticks "wrapped" between calls
+        to :meth:`ticks`.
+        """
+        raise NotImplementedError
 
     def _get_pi_info(self):
-        return None
+        return None  # pragma: no cover
 
     pi_info = property(
         lambda self: self._get_pi_info(),
@@ -134,7 +173,7 @@ class Factory(object):
 
         If the pins represented by this class are not *directly* attached to a
         Pi (e.g. the pin is attached to a board attached to the Pi, or the pins
-        are not on a Pi at all), this may return ``None``.
+        are not on a Pi at all), this may return :data:`None`.
         """)
 
 
@@ -171,7 +210,7 @@ class Pin(object):
     """
 
     def __repr__(self):
-        return "<Pin>"
+        return "<Pin>"  # pragma: no cover
 
     def close(self):
         """
@@ -214,12 +253,10 @@ class Pin(object):
         self.pull = pull
 
     def _get_function(self):
-        return "input"
+        raise NotImplementedError
 
     def _set_function(self, value):
-        if value != "input":
-            raise PinInvalidFunction(
-                "Cannot set the function of pin %r to %s" % (self, value))
+        raise NotImplementedError
 
     function = property(
         lambda self: self._get_function(),
@@ -237,10 +274,11 @@ class Pin(object):
         """)
 
     def _get_state(self):
-        return 0
+        raise NotImplementedError
 
     def _set_state(self, value):
-        raise PinSetInput("Cannot set the state of input pin %r" % self)
+        raise PinSetInput(  # pragma: no cover
+            "Cannot set the state of pin %r" % self)
 
     state = property(
         lambda self: self._get_state(),
@@ -259,8 +297,8 @@ class Pin(object):
 
         Descendents which implement analog, or analog-like capabilities can
         return values between 0 and 1. For example, pins implementing PWM
-        (where :attr:`frequency` is not ``None``) return a value between 0.0
-        and 1.0 representing the current PWM duty cycle.
+        (where :attr:`frequency` is not :data:`None`) return a value between
+        0.0 and 1.0 representing the current PWM duty cycle.
 
         If a pin is currently configured for input, and an attempt is made to
         set this attribute, :exc:`PinSetInput` will be raised. If an invalid
@@ -269,10 +307,11 @@ class Pin(object):
         """)
 
     def _get_pull(self):
-        return 'floating'
+        return 'floating'  # pragma: no cover
 
     def _set_pull(self, value):
-        raise PinFixedPull("Cannot change pull-up on pin %r" % self)
+        raise PinFixedPull(  # pragma: no cover
+            "Cannot change pull-up on pin %r" % self)
 
     pull = property(
         lambda self: self._get_pull(),
@@ -289,30 +328,33 @@ class Pin(object):
         """)
 
     def _get_frequency(self):
-        return None
+        return None  # pragma: no cover
 
     def _set_frequency(self, value):
         if value is not None:
-            raise PinPWMUnsupported("PWM is not supported on pin %r" % self)
+            raise PinPWMUnsupported(  # pragma: no cover
+                "PWM is not supported on pin %r" % self)
 
     frequency = property(
         lambda self: self._get_frequency(),
         lambda self, value: self._set_frequency(value),
         doc="""\
-        The frequency (in Hz) for the pin's PWM implementation, or ``None`` if
-        PWM is not currently in use. This value always defaults to ``None`` and
-        may be changed with certain pin types to activate or deactivate PWM.
+        The frequency (in Hz) for the pin's PWM implementation, or :data:`None`
+        if PWM is not currently in use. This value always defaults to
+        :data:`None` and may be changed with certain pin types to activate or
+        deactivate PWM.
 
         If the pin does not support PWM, :exc:`PinPWMUnsupported` will be
-        raised when attempting to set this to a value other than ``None``.
+        raised when attempting to set this to a value other than :data:`None`.
         """)
 
     def _get_bounce(self):
-        return None
+        return None  # pragma: no cover
 
     def _set_bounce(self, value):
-        if value is not None:
-            raise PinEdgeDetectUnsupported("Edge detection is not supported on pin %r" % self)
+        if value is not None:  # pragma: no cover
+            raise PinEdgeDetectUnsupported(
+                "Edge detection is not supported on pin %r" % self)
 
     bounce = property(
         lambda self: self._get_bounce(),
@@ -320,7 +362,7 @@ class Pin(object):
         doc="""\
         The amount of bounce detection (elimination) currently in use by edge
         detection, measured in seconds. If bounce detection is not currently in
-        use, this is ``None``.
+        use, this is :data:`None`.
 
         For example, if :attr:`edges` is currently "rising", :attr:`bounce` is
         currently 5/1000 (5ms), then the waveform below will only fire
@@ -349,10 +391,11 @@ class Pin(object):
         """)
 
     def _get_edges(self):
-        return 'none'
+        return 'none'  # pragma: no cover
 
     def _set_edges(self, value):
-        raise PinEdgeDetectUnsupported("Edge detection is not supported on pin %r" % self)
+        raise PinEdgeDetectUnsupported(  # pragma: no cover
+            "Edge detection is not supported on pin %r" % self)
 
     edges = property(
         lambda self: self._get_edges(),
@@ -378,10 +421,11 @@ class Pin(object):
         """)
 
     def _get_when_changed(self):
-        return None
+        return None  # pragma: no cover
 
     def _set_when_changed(self, value):
-        raise PinEdgeDetectUnsupported("Edge detection is not supported on pin %r" % self)
+        raise PinEdgeDetectUnsupported(  # pragma: no cover
+            "Edge detection is not supported on pin %r" % self)
 
     when_changed = property(
         lambda self: self._get_when_changed(),
@@ -389,14 +433,27 @@ class Pin(object):
         doc="""\
         A function or bound method to be called when the pin's state changes
         (more specifically when the edge specified by :attr:`edges` is detected
-        on the pin). The function or bound method must take no parameters.
+        on the pin). The function or bound method must accept two parameters:
+        the first will report the ticks (from :meth:`Factory.ticks`) when
+        the pin's state changed, and the second will report the pin's current
+        state.
+
+        .. warning::
+
+            Depending on hardware support, the state is *not guaranteed to be
+            accurate*. For instance, many GPIO implementations will provide
+            an interrupt indicating when a pin's state changed but not what it
+            changed to. In this case the pin driver simply reads the pin's
+            current state to supply this parameter, but the pin's state may
+            have changed *since* the interrupt. Exercise appropriate caution
+            when relying upon this parameter.
 
         If the pin does not support edge detection, attempts to set this
         property will raise :exc:`PinEdgeDetectUnsupported`.
         """)
 
 
-class SPI(object):
+class SPI(Device):
     """
     Abstract interface for `Serial Peripheral Interface`_ (SPI)
     implementations. Descendents *must* override the following methods:
@@ -429,7 +486,7 @@ class SPI(object):
         half-duplex communication. See :meth:`transfer` for full duplex
         communication.
         """
-        return self.transfer((0,) * n)
+        return self.transfer([0] * n)
 
     def write(self, data):
         """
@@ -463,14 +520,14 @@ class SPI(object):
     @property
     def clock_polarity(self):
         """
-        The polarity of the SPI clock pin. If this is ``False`` (the default),
-        the clock pin will idle low, and pulse high. Setting this to ``True``
-        will cause the clock pin to idle high, and pulse low. On many data
-        sheets this is documented as the CPOL value.
+        The polarity of the SPI clock pin. If this is :data:`False` (the
+        default), the clock pin will idle low, and pulse high. Setting this to
+        :data:`True` will cause the clock pin to idle high, and pulse low. On
+        many data sheets this is documented as the CPOL value.
 
         The following diagram illustrates the waveform when
-        :attr:`clock_polarity` is ``False`` (the default), equivalent to CPOL
-        0:
+        :attr:`clock_polarity` is :data:`False` (the default), equivalent to
+        CPOL 0:
 
         .. code-block:: text
 
@@ -482,7 +539,7 @@ class SPI(object):
             idle       off     off     off     off     off     off       idle
 
         The following diagram illustrates the waveform when
-        :attr:`clock_polarity` is ``True``, equivalent to CPOL 1:
+        :attr:`clock_polarity` is :data:`True`, equivalent to CPOL 1:
 
         .. code-block:: text
 
@@ -502,17 +559,17 @@ class SPI(object):
     @property
     def clock_phase(self):
         """
-        The phase of the SPI clock pin. If this is ``False`` (the default),
+        The phase of the SPI clock pin. If this is :data:`False` (the default),
         data will be read from the MISO pin when the clock pin activates.
-        Setting this to ``True`` will cause data to be read from the MISO pin
-        when the clock pin deactivates. On many data sheets this is documented
-        as the CPHA value. Whether the clock edge is rising or falling when the
-        clock is considered activated is controlled by the
+        Setting this to :data:`True` will cause data to be read from the MISO
+        pin when the clock pin deactivates. On many data sheets this is
+        documented as the CPHA value. Whether the clock edge is rising or
+        falling when the clock is considered activated is controlled by the
         :attr:`clock_polarity` attribute (corresponding to CPOL).
 
         The following diagram indicates when data is read when
-        :attr:`clock_polarity` is ``False``, and :attr:`clock_phase` is
-        ``False`` (the default), equivalent to CPHA 0:
+        :attr:`clock_polarity` is :data:`False`, and :attr:`clock_phase` is
+        :data:`False` (the default), equivalent to CPHA 0:
 
         .. code-block:: text
 
@@ -522,14 +579,14 @@ class SPI(object):
             ----'   `---'   `---'   `---'   `---'   `---'   `---'   `-------
                 :       :       :       :       :       :       :
             MISO---.   ,---.   ,---.   ,---.   ,---.   ,---.   ,---.
-              /     \ /     \ /     \ /     \ /     \ /     \ /     \\
+              /     \\ /     \\ /     \\ /     \\ /     \\ /     \\ /     \\
             -{  Bit  X  Bit  X  Bit  X  Bit  X  Bit  X  Bit  X  Bit  }------
-              \     / \     / \     / \     / \     / \     / \     /
+              \\     / \\     / \\     / \\     / \\     / \\     / \\     /
                `---'   `---'   `---'   `---'   `---'   `---'   `---'
 
         The following diagram indicates when data is read when
-        :attr:`clock_polarity` is ``False``, but :attr:`clock_phase` is
-        ``True``, equivalent to CPHA 1:
+        :attr:`clock_polarity` is :data:`False`, but :attr:`clock_phase` is
+        :data:`True`, equivalent to CPHA 1:
 
         .. code-block:: text
 
@@ -539,9 +596,9 @@ class SPI(object):
             ----'   `---'   `---'   `---'   `---'   `---'   `---'   `-------
                     :       :       :       :       :       :       :
             MISO   ,---.   ,---.   ,---.   ,---.   ,---.   ,---.   ,---.
-                  /     \ /     \ /     \ /     \ /     \ /     \ /     \\
+                  /     \\ /     \\ /     \\ /     \\ /     \\ /     \\ /     \\
             -----{  Bit  X  Bit  X  Bit  X  Bit  X  Bit  X  Bit  X  Bit  }--
-                  \     / \     / \     / \     / \     / \     / \     /
+                  \\     / \\     / \\     / \\     / \\     / \\     / \\     /
                    `---'   `---'   `---'   `---'   `---'   `---'   `---'
         """
         return bool(self.clock_mode & 1)
@@ -551,10 +608,11 @@ class SPI(object):
         self.clock_mode = self.clock_mode & (~1) | bool(value)
 
     def _get_clock_mode(self):
-        raise NotImplementedError
+        raise NotImplementedError  # pragma: no cover
 
     def _set_clock_mode(self, value):
-        raise SPIFixedClockMode("clock_mode cannot be changed on %r" % self)
+        raise SPIFixedClockMode(  # pragma: no cover
+            "clock_mode cannot be changed on %r" % self)
 
     clock_mode = property(
         lambda self: self._get_clock_mode(),
@@ -581,24 +639,25 @@ class SPI(object):
         """)
 
     def _get_lsb_first(self):
-        return False
+        return False  # pragma: no cover
 
     def _set_lsb_first(self, value):
-        raise SPIFixedBitOrder("lsb_first cannot be changed on %r" % self)
+        raise SPIFixedBitOrder(  # pragma: no cover
+            "lsb_first cannot be changed on %r" % self)
 
     lsb_first = property(
         lambda self: self._get_lsb_first(),
         lambda self, value: self._set_lsb_first(value),
         doc="""\
         Controls whether words are read and written LSB in (Least Significant
-        Bit first) order. The default is ``False`` indicating that words are
-        read and written in MSB (Most Significant Bit first) order.
+        Bit first) order. The default is :data:`False` indicating that words
+        are read and written in MSB (Most Significant Bit first) order.
         Effectively, this controls the `Bit endianness`_ of the connection.
 
         The following diagram shows the a word containing the number 5 (binary
         0101) transmitted on MISO with :attr:`bits_per_word` set to 4, and
-        :attr:`clock_mode` set to 0, when :attr:`lsb_first` is ``False`` (the
-        default):
+        :attr:`clock_mode` set to 0, when :attr:`lsb_first` is :data:`False`
+        (the default):
 
         .. code-block:: text
 
@@ -613,7 +672,7 @@ class SPI(object):
                 :       :       :       :
                MSB                     LSB
 
-        And now with :attr:`lsb_first` set to ``True`` (and all other
+        And now with :attr:`lsb_first` set to :data:`True` (and all other
         parameters the same):
 
         .. code-block:: text
@@ -633,22 +692,23 @@ class SPI(object):
         """)
 
     def _get_select_high(self):
-        return False
+        return False  # pragma: no cover
 
     def _set_select_high(self, value):
-        raise SPIFixedSelect("select_high cannot be changed on %r" % self)
+        raise SPIFixedSelect(  # pragma: no cover
+            "select_high cannot be changed on %r" % self)
 
     select_high = property(
         lambda self: self._get_select_high(),
         lambda self, value: self._set_select_high(value),
         doc="""\
-        If ``False`` (the default), the chip select line is considered active
-        when it is pulled low. When set to ``True``, the chip select line is
-        considered active when it is driven high.
+        If :data:`False` (the default), the chip select line is considered
+        active when it is pulled low. When set to :data:`True`, the chip select
+        line is considered active when it is driven high.
 
         The following diagram shows the waveform of the chip select line, and
-        the clock when :attr:`clock_polarity` is ``False``, and
-        :attr:`select_high` is ``False`` (the default):
+        the clock when :attr:`clock_polarity` is :data:`False`, and
+        :attr:`select_high` is :data:`False` (the default):
 
         .. code-block:: text
 
@@ -662,7 +722,7 @@ class SPI(object):
                 |   |   |   |   |   |   |   |   |   |   |   |   |   |
             ----'   `---'   `---'   `---'   `---'   `---'   `---'   `-------
 
-        And when :attr:`select_high` is ``True``:
+        And when :attr:`select_high` is :data:`True`:
 
         .. code-block:: text
 
@@ -678,10 +738,11 @@ class SPI(object):
         """)
 
     def _get_bits_per_word(self):
-        return 8
+        return 8  # pragma: no cover
 
     def _set_bits_per_word(self, value):
-        raise SPIFixedWordSize("bits_per_word cannot be changed on %r" % self)
+        raise SPIFixedWordSize(  # pragma: no cover
+            "bits_per_word cannot be changed on %r" % self)
 
     bits_per_word = property(
         lambda self: self._get_bits_per_word(),
@@ -694,4 +755,23 @@ class SPI(object):
         Several implementations do not support non-byte-sized words.
         """)
 
+    def _get_rate(self):
+        return 100000  # pragma: no cover
 
+    def _set_rate(self, value):
+        raise SPIFixedRate(  # pragma: no cover
+            "rate cannot be changed on %r" % self)
+
+    rate = property(
+        lambda self: self._get_rate(),
+        lambda self, value: self._set_rate(value),
+        doc="""\
+        Controls the speed of the SPI interface in Hz (or baud).
+
+        Note that most software SPI implementations ignore this property, and
+        will raise :exc:`SPIFixedRate` if an attempt is made to set it, as they
+        have no rate control (they simply bit-bang as fast as possible because
+        typically this isn't very fast anyway, and introducing measures to
+        limit the rate would simply slow them down to the point of being
+        useless).
+        """)
