@@ -1,3 +1,12 @@
+# vim: set fileencoding=utf-8:
+#
+# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
+#
+# Copyright (c) 2015-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
+#
+# SPDX-License-Identifier: BSD-3-Clause
+
 from __future__ import (
     unicode_literals,
     absolute_import,
@@ -24,9 +33,10 @@ from ..exc import (
 
 class RPiGPIOFactory(LocalPiFactory):
     """
-    Uses the `RPi.GPIO`_ library to interface to the Pi's GPIO pins. This is
-    the default pin implementation if the RPi.GPIO library is installed.
-    Supports all features including PWM (via software).
+    Extends :class:`~gpiozero.pins.local.LocalPiFactory`. Uses the `RPi.GPIO`_
+    library to interface to the Pi's GPIO pins. This is the default pin
+    implementation if the RPi.GPIO library is installed. Supports all features
+    including PWM (via software).
 
     Because this is the default pin implementation you can use it simply by
     specifying an integer number for the pin in most operations, e.g.::
@@ -59,8 +69,8 @@ class RPiGPIOFactory(LocalPiFactory):
 
 class RPiGPIOPin(LocalPiPin):
     """
-    Pin implementation for the `RPi.GPIO`_ library. See :class:`RPiGPIOFactory`
-    for more information.
+    Extends :class:`~gpiozero.pins.local.LocalPiPin`. Pin implementation for
+    the `RPi.GPIO`_ library. See :class:`RPiGPIOFactory` for more information.
 
     .. _RPi.GPIO: https://pypi.python.org/pypi/RPi.GPIO
     """
@@ -220,4 +230,3 @@ class RPiGPIOPin(LocalPiPin):
 
     def _disable_event_detect(self):
         GPIO.remove_event_detect(self.number)
-
