@@ -1,3 +1,14 @@
+# vim: set fileencoding=utf-8:
+#
+# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
+#
+# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2020 Grzegorz Szymaszek <gszymaszek@short.pl>
+# Copyright (c) 2016-2019 Andrew Scheller <github@loowis.durge.org>
+# Copyright (c) 2016-2018 Ben Nuttall <ben@bennuttall.com>
+#
+# SPDX-License-Identifier: BSD-3-Clause
+
 from __future__ import (
     unicode_literals,
     print_function,
@@ -14,7 +25,7 @@ try:
 except ImportError:
     pass # py2's reduce is built-in
 
-from .exc import DeviceClosed, SPIBadChannel
+from .exc import DeviceClosed, SPIBadChannel, InputDeviceError
 from .devices import Device
 
 
@@ -108,7 +119,7 @@ class AnalogInputDevice(SPIDevice):
 
         pot = MCP3008(0)
         led = PWMLED(17)
-        led.source = pot.values
+        led.source = pot
 
     The :attr:`voltage` attribute reports values between 0.0 and *max_voltage*
     (which defaults to 3.3, the logic level of the GPIO pins).
@@ -263,7 +274,7 @@ class MCP3xx2(MCP3xxx):
         # the result with the final byte of output. A start "1" bit is then
         # transmitted, followed by the single/differential bit (M); 1 for
         # single-ended read, 0 for differential read. Next comes a single bit
-        # for channel (M) then the MSBF bit (L) which selects whether the data
+        # for channel (C) then the MSBF bit (L) which selects whether the data
         # will be read out in MSB form only (1) or whether LSB read-out will
         # occur after MSB read-out (0).
         #
@@ -544,4 +555,3 @@ class MCP3304(MCP33xx):
         if not 0 <= channel < 8:
             raise SPIBadChannel('channel must be between 0 and 7')
         super(MCP3304, self).__init__(channel, differential, max_voltage, **spi_args)
-
