@@ -1,3 +1,10 @@
+# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
+#
+# Copyright (c) 2017-2019 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2016 Stewart <stewart@adcock.org.uk>
+#
+# SPDX-License-Identifier: BSD-3-Clause
+
 from __future__ import (
     unicode_literals,
     absolute_import,
@@ -7,6 +14,8 @@ from __future__ import (
 str = type('')
 
 
+import os
+
 import pytest
 
 from gpiozerocli.pinout import main
@@ -33,3 +42,7 @@ def test_help(capsys):
         main(['pinout', '--help'])
     out, err = capsys.readouterr()
     assert 'GPIO pin-out' in out
+
+def test_execution(capsys, no_default_factory):
+    os.environ['GPIOZERO_PIN_FACTORY'] = 'mock'
+    assert main([]) == 0
